unit DelphiIEBand;

interface

uses ComObj,// TComObjectFactory, TComObject
     ShlObj,// IDeskBand, IContextMenu, IInputObject, IInputObjectSite
     ActiveX,// IPersist, IPersistStreamInit, IObjectWithSite, IOleCommandTarget
     Windows,// dword
     Classes,// TWndMethod
     DelphiIEBandFrm,
     Messages,// TMessage
     Shdocvw;

const
  DeskBand = '{00021492-0000-0000-C000-000000000046}';
  VerticalBand = '{00021493-0000-0000-C000-000000000046}';
  HorizontalBand = '{00021494-0000-0000-C000-000000000046}';

type
  TDelphiBandFactory = class(TComObjectFactory)
  private
    procedure AddKeys;
    procedure RemoveKeys;
  protected
    function GetCLSID:string; virtual; abstract;
    function GetCaption:string; virtual; abstract;
    function GetBandType:string; virtual; abstract;
    function IsToolband:boolean; virtual; abstract;
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

  TDelphiBand = class(TComObject, IDeskBand, IPersist, IPersistStreamInit, IObjectWithSite, IContextMenu, IInputObject)
  private
    FContextMenu:TStrings;
    MenuItems : Integer;
    HasFocus: Boolean;
    BandID: DWORD;
    SavedWndProc: twndmethod;
    ParentWnd: HWND;
    Site: IInputObjectSite;
    cmdTarget: IOleCommandTarget;
    BandForm: TDelphiIEBandForm;
  protected
    IE : IWebbrowser2;
  public
   // IDeskBand methods
    function GetBandInfo(dwBandID, dwViewMode: DWORD; var pdbi: TDeskBandInfo):
      HResult; stdcall;
    function ShowDW(fShow: BOOL): HResult; stdcall;
    function CloseDW(dwReserved: DWORD): HResult; stdcall;
    function ResizeBorderDW(var prcBorder: TRect; punkToolbarSite: IUnknown;
      fReserved: BOOL): HResult; stdcall;
    function GetWindow(out wnd: HWnd): HResult; stdcall;
    function ContextSensitiveHelp(fEnterMode: BOOL): HResult; stdcall;
   // IPersistStreamInit methods
    function InitNew: HResult; stdcall;
    function GetClassID(out classID: TCLSID): HResult; stdcall;
    function IsDirty: HResult; stdcall;
    function Load(const stm: IStream): HResult; stdcall;
    function Save(const stm: IStream; fClearDirty: BOOL): HResult; stdcall;
    function GetSizeMax(out cbSize: Largeint): HResult; stdcall;
   // IObjectWithSite methods
    function SetSite(const pUnkSite: IUnknown): HResult; stdcall;
    function GetSite(const riid: TIID; out site: IUnknown): HResult; stdcall;
    // IContextMenu methods
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT; pszName: LPSTR; cchMax: UINT): HResult; stdcall;
    /// IInputObject methods
    function UIActivateIO(fActivate: BOOL; var lpMsg: TMsg): HResult; stdcall;
    function HasFocusIO: HResult; stdcall;
    function TranslateAcceleratorIO(var lpMsg: TMsg): HResult; stdcall;

    procedure FocusChange(bHasFocus: Boolean);
    procedure UpdateBandInfo;
    procedure BandWndProc(var Message: TMessage);

    destructor Destroy; override;
  protected
    // customization. the following must be overwritten for each new band
    procedure ContextMenuFired(AIndex:integer); virtual;
    function GetContextMenu:TStrings; virtual;
    function GetCLSID:string; virtual; abstract;
    function GetCaption:string; virtual; abstract;
    function GetBandType:string; virtual; abstract;
    function IsToolband:boolean; virtual; abstract;
    function IsFixed:boolean; virtual; abstract;// some bands are fixed size, since it doens't make sense resizing them
    function GetFormClass:TDelphiIEBandFormClass; virtual; abstract;
  end;

  TDelphiBandClass=class of TDelphiBand;
  
implementation

uses Registry, SysUtils;

procedure TDelphiBandFactory.UpdateRegistry(Register: Boolean);
begin
  inherited UpdateRegistry(Register);
  if Register then
    AddKeys
  else
    RemoveKeys;
end;

procedure TDelphiBandFactory.AddKeys;
var S: string;
begin
  S := GetCLSID;
  with TRegistry.Create do
  try
// http://support.microsoft.com/support/kb/articles/Q247/7/05.ASP   ->
    if GetBandType <> DeskBand then
    begin
      DeleteKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Discardable\PostSetup\Component Categories\' + VerticalBand + '\Enum');
      DeleteKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Discardable\PostSetup\Component Categories\' + HorizontalBand + '\Enum');
    end;

    RootKey := HKEY_CLASSES_ROOT;
    if OpenKey('CLSID\' + S, True) then
    begin
      WriteString('', '&'+GetCaption);
      CloseKey;
    end;
    if OpenKey('CLSID\' + S + '\InProcServer32', True) then
    begin
      WriteString('ThreadingModel', 'Apartment');
      CloseKey;
    end;
    if OpenKey('CLSID\' + S + '\Implemented Categories\' + GetBandType, True)
      then CloseKey;
    if IsToolBand then
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('SOFTWARE\Microsoft\Internet Explorer\Toolbar', True) then
      begin
        WriteString(S, '');
        CloseKey;
      end;
    end;
  finally
    Free;
  end;
end;

procedure TDelphiBandFactory.RemoveKeys;
var S: string;
begin
  S := GetCLSID;
  with TRegistry.Create do
  try
    RootKey := HKEY_CLASSES_ROOT;

    // http://support.microsoft.com/support/kb/articles/Q214/8/42.ASP ->
    if GetBandType = DeskBand then
      DeleteKey('Component Categories\' + DeskBand + '\Enum');

    DeleteKey('CLSID\' + S + '\Implemented Categories\' + GetBandType);
    DeleteKey('CLSID\' + S + '\InProcServer32');
    DeleteKey('CLSID\' + S);
    Closekey;
    if IsToolBand then
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey('Software\Microsoft\Internet Explorer\Toolbar', FALSE);
      DeleteValue(s);
      CloseKey;
    end;
  finally
    Free;
  end;
end;

{************************************************************}

procedure TDelphiBand.UpdateBandInfo;
(*
Band objects can send commands to their container.
Two commands are supported:
DBID_BANDINFOCHANGED
   The band's information has changed. The container will call the band
   object's GetBandInfo method to request the updated information.
DBID_MAXIMIZEBAND
The container will maximize the band.
*)
var
  vain, vaOut: OleVariant;
  PtrGuid: PGUID;
begin
  vaIn := Variant(BandID);
  New(PtrGUID);
  PtrGUID^ := IDESKBAND;
  cmdTarget.Exec(PtrGUID, DBID_BANDINFOCHANGED, OLECMDEXECOPT_DODEFAULT, vaIn, vaOut);
  Dispose(PtrGUID);
end;


function TDelphiBand.GetBandInfo(dwBandID, dwViewMode: DWORD; var pdbi: TDeskBandInfo):
  HResult;
// Retrieves the information for the band object.
var Caption:string;
begin
  BandId := dwBandID;
  (*
DBIM_MINSIZE:
The minimum size of the band object.
The minimum width is placed in the x member
and the minimum height is placed in the y member.

NB: In vertical bands is y=width e.g.:
    pdbi.ptMinSize.y:=Bandform.Width;
*)

  if (pdbi.dwMask or DBIM_MINSIZE) <> 0 then
  begin
    if GetBandType=VerticalBand then
      pdbi.ptMinSize.y := Bandform.width
    else
      pdbi.ptMinSize.y := Bandform.height;
    pdbi.ptMinSize.x := 0;//Bandform.width;
  end;

(*
DBIM_MAXSIZE:
The maximum size of the band object.
The maximum height is placed in the y member
and the x member is ignored. If there is no
limit for the maximum height, -1 should be used.
*)
  if (pdbi.dwMask or DBIM_MAXSIZE) <> 0 then
  begin
    if not IsFixed then
    begin
      pdbi.ptMaxSize.x := -1;
      pdbi.ptMaxSize.y := -1;
    end
      else
    begin
      if GetBandType=VerticalBand then
      begin
        pdbi.ptMaxSize.x := Bandform.height;
        pdbi.ptMaxSize.y := Bandform.width;
      end
        else
      begin
        pdbi.ptMaxSize.x := Bandform.width;
        pdbi.ptMaxSize.y := Bandform.height;
      end;
    end;
  end;

(*
DBIM_INTEGRAL:
the sizing step value of the band object.
The vertical step value is placed in the y member,
and the x member is ignored. The step value determines
in what increments the band will be resized.
This member is ignored if dwModeFlags does not
contain DBIMF_VARIABLEHEIGHT.
*)
  if (pdbi.dwMask or DBIM_INTEGRAL) <> 0 then
  begin
    pdbi.ptIntegral.x := 1;
    pdbi.ptIntegral.y := 1;
  end;


(*
DBIM_ACTUAL:
The ideal size of the band object. The ideal width
is placed in the x member and the ideal height is placed
in the y member. The band container will attempt to use
these values, but the band is not guaranteed to be this size.
*)
  if (pdbi.dwMask or DBIM_ACTUAL) <> 0 then
  begin
    pdbi.ptActual.x := Bandform.Width;
    pdbi.ptActual.y := bandform.Height;
  end;

(*
DBIM_MODEFLAGS:
A value that receives a set of flags that define the mode of
operation for the band object. This must be one or a combination
of the following values:

DBIMF_NORMAL:  The band is normal in all respects. The other mode flags modify this flag.
DBIMF_VARIABLEHEIGHT:  The height of the band object can be changed.
        The ptIntegral member defines the step value by which the band object can be resized.
DBIMF_DEBOSSED:  The band object is displayed with a sunken appearance.
DBIMF_BKCOLOR:  The band will be displayed with the background color specified in crBkgnd
*)
  if (pdbi.dwMask or DBIM_MODEFLAGS) <> 0 then
    pdbi.dwModeFlags := DBIMF_NORMAL;

(*
DBIM_BKCOLOR:
A value that eceives the background color of the band.
This member is ignored if dwModeFlags does not contain the DBIMF_BKCOLOR flag.
*)
  if (pdbi.dwMask or DBIM_BKCOLOR) <> 0 then
    pdbi.dwMask := pdbi.dwMask and (not DBIM_BKCOLOR);


(*
DBIM_TITLE:
the title of the band.
*)
  if (Pdbi.dwMask and DBIM_TITLE) = DBIM_TITLE then
  begin
    Caption:=GetCaption;
    FillChar(pdbi.wszTitle, SizeOf(GetCaption) + 1, ' ');
    StringToWideChar(Caption, @pdbi.wszTitle, Length(Caption) + 1);
  end;
  Result := NOERROR;
end;


function TDelphiBand.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult;
var Bean, BeanPile:cardinal;
begin
//Add Menuitems here in reverse order:
  if not assigned(FContextMenu) then
    FContextMenu:=GetContextMenu;
  if not assigned(FContextMenu) then
  begin
    MenuItems:=0;
    result:=0;
    exit;
  end;
  BeanPile:=FContextMenu.count;
  for Bean:=BeanPile-1 downto 0 do
    InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, idCmdFirst + Bean, pchar(FContextMenu[Bean]));
  MenuItems := FContextMenu.count;
// Return number of items added:
  Result := MenuItems;
end;


function TDelphiBand.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
begin
  if (HiWord(Integer(lpici.lpVerb)) <> 0) or (LoWord(lpici.lpVerb) > MenuItems-1) then
  begin
    Result := E_FAIL;
    Exit;
  end;
  ContextMenuFired(LoWord(lpici.lpVerb));
  Result := NO_ERROR;
end;



procedure TDelphiBand.BandWndProc(var Message: TMessage);
begin
// Notify the browser when the band has focus so Accelerator
// keys are translated.
  if (Message.Msg = WM_PARENTNOTIFY)  then
  begin
    Hasfocus:=True;
    FocusChange(True);
  end;
  SavedWndProc(Message);
end;

function TDelphiBand.GetWindow(out wnd: HWnd): HResult;
begin
  if not Assigned(BandForm) then
  begin
    BandForm := GetFormClass.CreateParented(ParentWnd);
    BandForm.FIE:=IE;
    BandForm.FCLSID:=StringToGUID(GetCLSID);
  end;
  Wnd := Bandform.Handle;
  SavedWndProc := Bandform.WindowProc;
  Bandform.WindowProc := BandWndProc;
  Result := S_OK;
end;

procedure TDelphiBand.FocusChange(bHasFocus: Boolean);
begin
// Informs the browser that the focus has changed.
  if (Site <> nil) then
    Site.OnFocusChangeIS(Self, bHasFocus);
end;

function TDelphiBand.TranslateAcceleratorIO(var lpMsg: TMsg): HResult;
// Passes keyboard accelerators to the object.

begin
  if (lpMsg.WParam <> VK_TAB) then
  begin
    TranslateMessage(lpMSg);
    DispatchMessage(lpMsg);
    Result := S_OK;
  end
  else
    Result := S_FALSE;
end;


function TDelphiBand.HasFocusIO: HResult;
// Determines if one of the object's windows has the keyboard focus.
begin
  Result:=Integer(not HasFocus);
end;


function TDelphiBand.UIActivateIO(fActivate: BOOL;
  var lpMsg: TMsg): HResult;
// Activates or deactivates the object.
begin
  Hasfocus:=fActivate;
  if HasFocus then
    Bandform.SetFocus;
  Result := S_OK;
end;


function TDelphiBand.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT; pszName: LPSTR; cchMax: UINT): HResult;
begin
  Result := NOERROR;
end;


function TDelphiBand.SetSite(const pUnkSite: IUnknown): HResult;
// When the user selects an Explorer Bar, the container calls
// the corresponding band object's SetSite method. The punkSite
// parameter will be set to the site's IUnknown pointer.
begin
//If the pointer passed to SetSite is set to Nil, the band is being removed.
//SetSite can return S_OK. ->
  if Assigned(pUnkSite) then
  begin
// Store the pointer to this interface for use later. ->
    Site := pUnkSite as IInputObjectSite;
//Call GetWindow to obtain the parent window's handle,
//and save it for future use. ->
    (pUnkSite as IOleWindow).GetWindow(ParentWnd);
// Need IOleCommandTarget if you want to send commands to the container
// (see UpdateBandInfo) ->
    cmdTarget := pUnkSite as IOleCommandTarget;
//  Get a connection to IE's browser-window ->
    (CmdTarget as IServiceProvider).QueryService(IWebbrowserApp, IWebbrowser2, IE);
  end;
  Result := S_OK;
end;

function TDelphiBand.GetSite(const riid: TIID; out site: IUnknown): HResult;
// Retrieves the last site set with SetSite.
begin
  if Assigned(Site) then
    Result := Site.QueryInterface(riid, site)
  else
    Result := E_FAIL;
end;

function TDelphiBand.GetClassID(out classID: TCLSID): HResult;
begin
  classID := StringToGUID(GetCLSID);
  Result := S_OK;
end;


function TDelphiBand.CloseDW(dwReserved: DWORD): HResult;
begin
  if BandForm <> nil then
    BandForm.Destroy;
  Result := S_OK;
end;


function TDelphiBand.ContextSensitiveHelp(fEnterMode: BOOL): HResult;
begin
  Result := E_NOTIMPL;
end;

function TDelphiBand.ShowDW(fShow: BOOL): HResult;
begin
  Hasfocus:=fShow;
  FocusChange(fShow);
  Result := S_OK;
end;


function TDelphiBand.ResizeBorderDW(var prcBorder: TRect; punkToolbarSite: IUnknown;
  fReserved: BOOL): HResult;
begin
  Result := E_NOTIMPL;
end;


function TDelphiBand.IsDirty: HResult;
begin
  Result := S_FALSE;
end;

function TDelphiBand.Load(const stm: IStream): HResult;
begin
  Result := S_OK;
end;

function TDelphiBand.Save(const stm: IStream; fClearDirty: BOOL): HResult;
begin
  Result := S_OK;
end;

function TDelphiBand.GetSizeMax(out cbSize: Largeint): HResult;
begin
  Result := E_NOTIMPL;
end;

function TDelphiBand.InitNew: HResult;
begin
  Result := E_NOTIMPL;
end;

procedure TDelphiBand.ContextMenuFired(AIndex: integer);
begin
// do nothing.
end;

function TDelphiBand.GetContextMenu: TStrings;
begin
  result:=nil;
end;

destructor TDelphiBand.Destroy;
begin
  freeandnil(FContextMenu);
  inherited;
end;

end.
